//
//  tLBulletHelloWorldPlugIn.m
//  tLBulletHelloWorld
//
//  Created by pascal on 5/5/11.
//  Copyright (c) 2011 t-linked.com . All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
//#import <OpenGL/CGLMacro.h>
#import <OpenGL/gluMacro.h>
#import "tLBulletHelloWorldPlugIn.h"
#include "TLUtils.h"

#define	kQCPlugIn_Name				@"tLBulletHelloWorld"
#define	kQCPlugIn_Description		@"tLBulletHelloWorld description"

#define kRenderFPS		60.0
static float		width;
static float		height;
static float		aspect;

static float worldDimWidth ;
static float worldDimHeight ;
static float worldDimDepth ;

static int spheresCount;
static int boxesCount;

// Bullet Physic Engine 
#include <BulletDynamics/btBulletDynamicsCommon.h>
#include <LinearMath/btAlignedObjectArray.h>

#define WORLD_WIDTH		300
#define WORLD_HEIGHT	100
#define WORLD_DEPTH	    200

#define MAX_SPHERES_COUNT 200
#define MAX_BOXES_COUNT 200

#define NODE_MIN_DIM 10
#define NODE_MAX_DIM 40


#define MIN_MASS	1
#define MAX_MASS	4


// Bullet Physics Objects 
btDiscreteDynamicsWorld * bt_dynamicsWorld;

btBroadphaseInterface*	bt_broadphase;

btCollisionDispatcher*	bt_dispatcher;

btConstraintSolver*	bt_solver;

btDefaultCollisionConfiguration* bt_collisionConfiguration;

btRigidBody* bt_sphereBody[MAX_SPHERES_COUNT];
btRigidBody* bt_boxBody[MAX_BOXES_COUNT];

btRigidBody* bt_leftWallBody;
btRigidBody* bt_rightWallBody;
btRigidBody* bt_frontWallBody;
btRigidBody* bt_backWallBody;
btRigidBody* bt_floorWallBody;
btRigidBody* bt_ceillingWallBody;

@implementation tLBulletHelloWorldPlugIn

/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

@dynamic inputRenderWidth, inputRenderHeight ;
@dynamic inputWorldDimWidth, inputWorldDimHeight, inputWorldDimDepth;
@dynamic inputGravityX, inputGravityY, inputGravityZ;
@dynamic inputSpheresCount, inputBoxesCount;
@dynamic inputSpheresColor;
@dynamic inputBoxesColor;
@dynamic inputBoundingArea;



+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeConsumer;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeTimeBase;
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	Release any resources created in -init.
	*/
	
	[super dealloc];
}

+ (NSArray*) plugInKeys
{
	/*
	Return a list of the KVC keys corresponding to the internal settings of the plug-in.
	*/
	
	return nil;
}

- (id) serializedValueForKey:(NSString*)key;
{
	/*
	Provide custom serialization for the plug-in internal settings that are not values complying to the <NSCoding> protocol.
	The return object must be nil or a PList compatible i.e. NSString, NSNumber, NSDate, NSData, NSArray or NSDictionary.
	*/
	
	return [super serializedValueForKey:key];
}

- (void) setSerializedValue:(id)serializedValue forKey:(NSString*)key
{
	/*
	Provide deserialization for the plug-in internal settings that were custom serialized in -serializedValueForKey.
	Deserialize the value, then call [self setValue:value forKey:key] to set the corresponding internal setting of the plug-in instance to that deserialized value.
	*/
	
	[super setSerializedValue:serializedValue forKey:key];
}

- (QCPlugInViewController*) createViewController
{
	/*
	Return a new QCPlugInViewController to edit the internal settings of this plug-in instance.
	You can return a subclass of QCPlugInViewController if necessary.
	*/
	
	return [[QCPlugInViewController alloc] initWithPlugIn:self viewNibName:@"Settings"];
}

@end

@implementation tLBulletHelloWorldPlugIn (Execution)

/// --- Rendering Methods ---

- (void) initGLScene: (CGLContextObj) cgl_ctx
{
	// lock the Gl context 
	//CGLLockContext(cgl_ctx);
	
	// Enable a single OpenGL light.
	GLfloat mat_shininess[] = { 5.0 };
	GLfloat mat_specular[] = { 0.5, 0.5, 0.5, 1.0 };
	glShadeModel(GL_SMOOTH);
	
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
	
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHT0);
	
	// Enable back-face culling 
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);
	
	// Enable Depth test
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,  GL_ONE_MINUS_SRC_ALPHA);
	
	// Enable Depth test
	glEnable(GL_DEPTH_TEST);
	
	//  Unlock the GL context
	//CGLUnlockContext(cgl_ctx);
}

- (void) renderGLScene : (CGLContextObj) cgl_ctx
{
	// Lock the Gl context
	CGLLockContext(cgl_ctx);
	
	// save our current GL state
	glPushAttrib(GL_COLOR_BUFFER_BIT | GL_TRANSFORM_BIT | GL_VIEWPORT_BIT);
	
	//Set OpenGl state
	{		
		// Clear to black
		glClearColor(0.0,0.0,0.0,1.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
		// Set light position 
		GLfloat light_position[] = { 0, 0 , worldDimDepth*3 , 0.0 };
		glLightfv(GL_LIGHT0, GL_POSITION, light_position);
		
		//Save & configure projection matrix
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		
		gluPerspective(45, //view angle
					   aspect,	//aspect ratio
					   1.0, //near clip
					   width*3.f);//far clip
		
		
		//Save & configure model view matrix
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();
		gluLookAt(0.0, 0.0, width/aspect,  /* eye is at (0,0,5) */
				  0.0, 0.0, 0.0,      /* center is at (0,0,0) */
				  0.0, 1.0, 0.);      /* up is in positive Y direction */
		
	}
	
	// Draw bounding box
	if ( self.inputBoundingArea ) 
		[self drawBoundingBox:cgl_ctx];
	
	// Draw boxes 
	[self drawBoxes:cgl_ctx];
	
	// Draw boxes 
	[self drawSpheres:cgl_ctx];

	// Restore OpenGL states
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	
	// Restore OpenGl Attributs
	glPopAttrib();
	
	// Lock the gl contaxt
	CGLUnlockContext(cgl_ctx);
}


- (void) drawBoundingBox:(CGLContextObj) cgl_ctx;
{
	glColor4f(1.f, 1.f, 1.f, 1.f);

	// back wall	
	glBegin(GL_LINE_LOOP);
	{
		glVertex3f(-worldDimWidth/2.f, worldDimHeight/2.f, -worldDimDepth);
		glVertex3f(-worldDimWidth/2.f, -worldDimHeight/2.f, -worldDimDepth);
		glVertex3f( worldDimWidth/2.f, -worldDimHeight/2.f, -worldDimDepth);
		glVertex3f( worldDimWidth/2.f, worldDimHeight/2.f, -worldDimDepth);
	}
	glEnd();
	
	
	
	// right 
	glBegin(GL_LINE_LOOP);
	{
		glVertex3f(worldDimWidth/2.f, worldDimHeight/2.f, -worldDimDepth);
		glVertex3f(worldDimWidth/2.f, -worldDimHeight/2.f, -worldDimDepth);
		glVertex3f(worldDimWidth/2.f, -worldDimHeight/2.f, 0.f);
		glVertex3f(worldDimWidth/2.f, worldDimHeight/2.f, 0.f);
	}
	glEnd();
	
	// left wall
	glBegin(GL_LINE_LOOP);
	{
		glVertex3f(-worldDimWidth/2.f, worldDimHeight/2.f, 0.f);
		glVertex3f(-worldDimWidth/2.f, -worldDimHeight/2.f, 0.f);
		glVertex3f(-worldDimWidth/2.f, -worldDimHeight/2.f, -worldDimDepth);
		glVertex3f(-worldDimWidth/2.f, worldDimHeight/2.f, -worldDimDepth);
	}
	glEnd();
	
	// front wall
	glBegin(GL_LINE_LOOP);
	{
		glVertex3f(-worldDimWidth/2.f, -worldDimHeight/2.f, 0.f);
		glVertex3f(-worldDimWidth/2.f, worldDimHeight/2.f, 0.f);
		glVertex3f(worldDimWidth/2.f, worldDimHeight/2.f, 0.f);
		glVertex3f(worldDimWidth/2.f, -worldDimHeight/2.f, 0.f);
	}
	glEnd();
}

- (void) drawSpheres: (CGLContextObj) cgl_ctx
{
	// Enable Lighting
	glEnable(GL_LIGHTING);
	
	// Spheres color
	const CGFloat*    colorComponents;
	colorComponents = CGColorGetComponents(self.inputSpheresColor);
    glColor4f(colorComponents[0], colorComponents[1], colorComponents[2], colorComponents[3]);
	
	int i ; 
	for (i=0; i<spheresCount;i++)
	{
		if ( bt_sphereBody[i] != NULL)
		{
			btTransform trans;
			
			// Get rigidBody positions
			bt_sphereBody[i]->getMotionState()->getWorldTransform(trans);
			btVector3 position = trans.getOrigin();
			
			// Get rotation angle/axis
			btScalar angle = trans.getRotation().getAngle();
			btVector3 axis = trans.getRotation().getAxis();
			
			// Force Activation 
			if (!(bt_sphereBody[i]->isActive()))
				bt_sphereBody[i]->activate(true);
			
			// Draw Box
			glPushMatrix();
			{			
				// Translate
				glTranslatef(position.getX(), position.getY(), position.getZ());
				
				// Rotate
				glRotatef(angle*SIMD_DEGS_PER_RAD, axis.getX(), axis.getY(), axis.getZ() );
				
				// Get shape radius 
				btSphereShape* _sphere = (btSphereShape* )bt_sphereBody[i]->getCollisionShape();
			
				GLUquadricObj *quadratic;
				quadratic=gluNewQuadric();
				gluQuadricNormals(quadratic, GLU_SMOOTH);
				gluQuadricTexture(quadratic, GL_TRUE);
				gluSphere(quadratic, _sphere->getRadius(),15,15);
			}
			glPopMatrix();
		}
	}
	// Enable Lighting
	glDisable(GL_LIGHTING);
	
}

- (void) drawBoxes: (CGLContextObj) cgl_ctx
{
	// Enable Lighting
	glEnable(GL_LIGHTING);
	
	// Boxes color 
	const CGFloat*    colorComponents;
	colorComponents = CGColorGetComponents(self.inputBoxesColor);
    glColor4f(colorComponents[0], colorComponents[1], colorComponents[2], colorComponents[3]);

	
	int i ; 
	for (i=0; i<boxesCount;i++)
	{
		if ( bt_boxBody[i] != NULL)
		{
			btTransform trans;
			
			// Get rigidBody positions
			bt_boxBody[i]->getMotionState()->getWorldTransform(trans);
			btVector3 position = trans.getOrigin();
			
			// Get rotation angle/axis
			btScalar angle = trans.getRotation().getAngle();
			btVector3 axis = trans.getRotation().getAxis();
			
			// Force Activation 
			if (!(bt_boxBody[i]->isActive()))
				bt_boxBody[i]->activate(true);
			
			// Draw Box
			glPushMatrix();
			{
				// Translate
				glTranslatef(position.getX(), position.getY(), position.getZ());
				
				// Rotate
				glRotatef(angle*SIMD_DEGS_PER_RAD, axis.getX(), axis.getY(), axis.getZ() );
				
				// Scale 
				btBoxShape* box = (btBoxShape* )bt_boxBody[i]->getCollisionShape();
				btVector3 halfExtent = box->getHalfExtentsWithMargin();
				
				// Draw box
				{
					static int indices[36] = 
					{
						0,1,2,
						3,2,1,
						4,0,6,
						6,0,2,
						5,1,4,
						4,1,0,
						7,3,1,
						7,1,5,
						5,4,7,
						7,4,6,
						7,2,3,
						7,6,2
					};
					
					btVector3 vertices[8]={	
						btVector3(halfExtent[0],halfExtent[1],halfExtent[2]),
						btVector3(-halfExtent[0],halfExtent[1],halfExtent[2]),
						btVector3(halfExtent[0],-halfExtent[1],halfExtent[2]),	
						btVector3(-halfExtent[0],-halfExtent[1],halfExtent[2]),	
						btVector3(halfExtent[0],halfExtent[1],-halfExtent[2]),
						btVector3(-halfExtent[0],halfExtent[1],-halfExtent[2]),	
						btVector3(halfExtent[0],-halfExtent[1],-halfExtent[2]),	
						btVector3(-halfExtent[0],-halfExtent[1],-halfExtent[2])};

					glBegin (GL_TRIANGLES);
					int si=36;
					for (int i=0;i<si;i+=3)
					{
						const btVector3& v1 = vertices[indices[i]];;
						const btVector3& v2 = vertices[indices[i+1]];
						const btVector3& v3 = vertices[indices[i+2]];
						
						
						btVector3 normal = (v1-v3).cross(v1-v2);
						normal.normalize ();
						glNormal3f(-normal.getX(),-normal.getY(),-normal.getZ());
						glVertex3f (v1.x(), v1.y(), v1.z());
						glVertex3f (v2.x(), v2.y(), v2.z());
						glVertex3f (v3.x(), v3.y(), v3.z());
						
					}
					glEnd();
				}				
			}
			glPopMatrix();
			
		}
	}
	
	glDisable(GL_LIGHTING);
	
}

/// --- Physics Methods  ---
- (void) initPhysics
{
	///collision configuration contains default setup for memory, collision setup
	bt_collisionConfiguration = new btDefaultCollisionConfiguration();
	
	///use the default collision dispatcher. 
	bt_dispatcher = new	btCollisionDispatcher(bt_collisionConfiguration);	
	bt_broadphase = new btDbvtBroadphase();
	
	///the default constraint solver.
	btSequentialImpulseConstraintSolver* sol = new btSequentialImpulseConstraintSolver;
	bt_solver = sol;
	
	bt_dynamicsWorld = new btDiscreteDynamicsWorld(bt_dispatcher,bt_broadphase,bt_solver,bt_collisionConfiguration);
	
	bt_dynamicsWorld->setGravity(btVector3(0.f,0.f,0.f));
	
	[self createBoundingBody];
}

- (void) createBoundingBody
{
	btScalar mass(0.);
	btVector3 localInertia(0,0,0);
	
	btTransform transform;
	transform.setIdentity();
	
	///create a floor rigid bodie
	{
		btCollisionShape* floorShape = new btStaticPlaneShape(btVector3(0. ,1. ,0.) , 0. );
		
		transform.setOrigin(btVector3(0,-worldDimHeight/2.f,0));
		
		//rigidbody is static
		floorShape->calculateLocalInertia(mass,localInertia);
		
		// rigid body construction
		btDefaultMotionState* myMotionState = new btDefaultMotionState(transform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,floorShape,localInertia);
		bt_floorWallBody = new btRigidBody(rbInfo);
		
		//add the body to the dynamics world
		bt_dynamicsWorld->addRigidBody(bt_floorWallBody);
		
	}
	
	///create a ceilling rigid bodie
	{
		btCollisionShape* ceillingShape = new btStaticPlaneShape(btVector3(0.f ,-1.f ,0.f) , 0.f );
		
		transform.setOrigin(btVector3(0,worldDimHeight/2.f,0));
		
		//rigidbody is static
		ceillingShape->calculateLocalInertia(mass,localInertia);
		
		// rigid body construction
		btDefaultMotionState* myMotionState = new btDefaultMotionState(transform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,ceillingShape,localInertia);
		bt_ceillingWallBody = new btRigidBody(rbInfo);
		
		//add the body to the dynamics world
		bt_dynamicsWorld->addRigidBody(bt_ceillingWallBody);
		
	}
	
	///create a left wall rigid bodie
	{
		btCollisionShape* leftShape = new btStaticPlaneShape(btVector3(1. ,0. ,0.) , 0. );
		
		transform.setOrigin(btVector3(-worldDimWidth/2.f,0,0));
		
		//rigidbody is static
		leftShape->calculateLocalInertia(mass,localInertia);
		
		// rigid body construction
		btDefaultMotionState* myMotionState = new btDefaultMotionState(transform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,leftShape,localInertia);
		bt_leftWallBody = new btRigidBody(rbInfo);
		
		//add the body to the dynamics world
		bt_dynamicsWorld->addRigidBody(bt_leftWallBody);
		
	}
	
	///create a right wall rigid bodie
	{
		btCollisionShape* rightShape = new btStaticPlaneShape(btVector3(-1. ,0. ,0.) , 0. );
		
		transform.setOrigin(btVector3(worldDimWidth/2.f,0,0));
		
		//rigidbody is static
		rightShape->calculateLocalInertia(mass,localInertia);
		
		// rigid body construction
		btDefaultMotionState* myMotionState = new btDefaultMotionState(transform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,rightShape,localInertia);
		bt_rightWallBody = new btRigidBody(rbInfo);
		
		//add the body to the dynamics world
		bt_dynamicsWorld->addRigidBody(bt_rightWallBody);
		
	}
	
	///create a front wall rigid bodie
	{
		btCollisionShape* frontShape = new btStaticPlaneShape(btVector3(0. ,0. ,-1.) , 0. );
		
		transform.setOrigin(btVector3(0,0,0));
		
		//rigidbody is static
		frontShape->calculateLocalInertia(mass,localInertia);
		
		// rigid body construction
		btDefaultMotionState* myMotionState = new btDefaultMotionState(transform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,frontShape,localInertia);
		bt_frontWallBody = new btRigidBody(rbInfo);
		
		//add the body to the dynamics world
		bt_dynamicsWorld->addRigidBody(bt_frontWallBody);
		
	}
	
	///create a back wall rigid bodie
	{
		btCollisionShape* backShape = new btStaticPlaneShape(btVector3(0. ,0. ,1.) , 0. );
		
		transform.setOrigin(btVector3(0,0,-worldDimDepth));
		
		//rigidbody is static
		backShape->calculateLocalInertia(mass,localInertia);
		
		// rigid body construction
		btDefaultMotionState* myMotionState = new btDefaultMotionState(transform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,backShape,localInertia);
		bt_backWallBody = new btRigidBody(rbInfo);
		
		//add the body to the dynamics world
		bt_dynamicsWorld->addRigidBody(bt_backWallBody);
		
	}
	
}

- (void) updateWorldDim
{
	
	btTransform transform;
    transform.setIdentity();	
	
	//
	{	
		transform.setOrigin(btVector3(-worldDimWidth/2.f ,0. ,0.));
		btDefaultMotionState* letfNewMotionState = new btDefaultMotionState(transform);
		bt_leftWallBody->setMotionState(letfNewMotionState);
		
		transform.setOrigin(btVector3(worldDimWidth/2.f ,0. ,0.));
		btDefaultMotionState* rightNewMotionState = new btDefaultMotionState(transform);
		bt_rightWallBody->setMotionState(rightNewMotionState);
	}
	
	//
	{
		transform.setOrigin(btVector3(0. ,-worldDimHeight/2.f ,0.));
		btDefaultMotionState* flooNewMotionState = new btDefaultMotionState(transform);
		bt_floorWallBody->setMotionState(flooNewMotionState);
		
		transform.setOrigin(btVector3(0. ,worldDimHeight/2.f ,0.));
		btDefaultMotionState* ceillingNewMotionState = new btDefaultMotionState(transform);
		bt_ceillingWallBody->setMotionState(ceillingNewMotionState);
	}
	
	//
	{
		transform.setOrigin(btVector3(0.,0. ,0));
		btDefaultMotionState* frontNewMotionState = new btDefaultMotionState(transform);
		bt_frontWallBody->setMotionState(frontNewMotionState);
		
		transform.setOrigin(btVector3(0. ,0. ,-worldDimDepth));
		btDefaultMotionState* backNewMotionState = new btDefaultMotionState(transform);
		bt_backWallBody->setMotionState(backNewMotionState);
	}

}

//
- (void) addRandomSphere
{
	// Start up transform
	btTransform transform;
	transform.setIdentity();
	
	// Set up mass and inertia
	btScalar mass = tlRandomFloat(MIN_MASS,MAX_MASS);
	btScalar radius	= tlMap(mass, MIN_MASS, MAX_MASS, NODE_MIN_DIM/2., NODE_MAX_DIM/2.,true);
	
	btVector3 localInertia(0,0,0);
	
	// Create sphere shape
	btCollisionShape* colShape = new btSphereShape(btScalar(radius));
	
	// Set mass and local inertia 
	colShape->calculateLocalInertia(mass,localInertia);
	
	// Set start up transformation
	btVector3 position = btVector3 ( tlRandomFloat(-worldDimWidth/2.f + radius , worldDimWidth/2.f - radius), 
									worldDimHeight/2.f - radius ,
									tlRandomFloat(-worldDimDepth/2.f + radius , - radius)
									); 
	transform.setOrigin(position);
	
	
	// Create sphere rigid body 
	btDefaultMotionState* motionState = new btDefaultMotionState(transform);
	btRigidBody::btRigidBodyConstructionInfo rbInfo(mass, motionState, colShape, localInertia);
	btRigidBody* body = new btRigidBody(rbInfo);
	 
	
	// adds the rididbody to the world 
	bt_sphereBody[spheresCount]	 = body ; 
	bt_dynamicsWorld->addRigidBody(body);
	
	// increments spheresCount
	spheresCount++;
	
}

- (void) updateSpheresCount
{
	int _spheresCount = spheresCount;
	int newSpheresCount = (int) ( MAX(0 , MIN(MAX_SPHERES_COUNT, self.inputSpheresCount)));

	
	if ( newSpheresCount > _spheresCount)
	{
		for(int i=_spheresCount; i<newSpheresCount; i++) 
		{
			[self addRandomSphere];
		}
	}
	
	if ( newSpheresCount < _spheresCount)
	{
		for(int i=newSpheresCount ; i<_spheresCount; i++) 
		{
			
			if (bt_sphereBody[i]!= NULL)
			{
				delete bt_sphereBody[i]->getMotionState();
				delete bt_sphereBody[i]->getCollisionShape();
			}
			bt_dynamicsWorld->removeCollisionObject( bt_sphereBody[i]);
			bt_sphereBody[i] = NULL ;
			
			spheresCount -- ; 
			
		}
	}
}


//
- (void) addRandomBox
{
	// Start up transform
	btTransform transform;
	transform.setIdentity();
	
	// Get random sizes
	float w = tlRandomFloat(NODE_MIN_DIM, NODE_MAX_DIM);
	float h = tlRandomFloat(NODE_MIN_DIM, NODE_MAX_DIM);
	float d = tlRandomFloat(NODE_MIN_DIM, NODE_MAX_DIM);
	float v = w*h*d;
	
	float min_vol = NODE_MIN_DIM * NODE_MIN_DIM *NODE_MIN_DIM ;
	float max_vol = NODE_MAX_DIM * NODE_MAX_DIM *NODE_MAX_DIM ;
	
	// Set up mass and inertia
	btScalar mass = tlMap(v, min_vol, max_vol, MIN_MASS, MAX_MASS, true);
	btVector3 localInertia(0,0,0);
	
	// Create boxe shape
	btCollisionShape* colShape = new btBoxShape(btVector3(w/2.f ,h/2.f ,d/2.f));
	colShape->calculateLocalInertia(mass,localInertia);
	
	// Set start up transformation
	
	btVector3 position = btVector3 ( tlRandomFloat(-worldDimWidth/2.f + w/2.f , worldDimWidth/2.f - w/2.f), 
									worldDimHeight/2.f - h/2.f ,
									tlRandomFloat(-worldDimDepth/2.f + d/2.f , - d/2.f)
									); 
	transform.setOrigin(position);
	
	
	// Create sphere rigid body 
	btDefaultMotionState* motionState = new btDefaultMotionState(transform);
	btRigidBody::btRigidBodyConstructionInfo rbInfo(mass, motionState, colShape, localInertia);
	btRigidBody* body = new btRigidBody(rbInfo);
	
	
	// adds the rididbody to the world 
	bt_boxBody[boxesCount]	 = body ; 
	bt_dynamicsWorld->addRigidBody(body);
	
	// increments spheresCount
	boxesCount++;
	
}

- (void) updateBoxesCount
{
	int _boxesCount = boxesCount;
	int newBoxesCount = (int) ( MAX(0 , MIN(MAX_BOXES_COUNT, self.inputBoxesCount)));
	
	
	if ( newBoxesCount > _boxesCount)
	{
		for(int i=_boxesCount; i<newBoxesCount; i++) 
		{
			[self addRandomBox];
		}
	}
	
	if ( newBoxesCount < _boxesCount)
	{
		for(int i=newBoxesCount ; i<_boxesCount; i++) 
		{
			
			if (bt_boxBody[i]!= NULL)
			{
				delete bt_boxBody[i]->getMotionState();
				delete bt_boxBody[i]->getCollisionShape();
			}
			bt_dynamicsWorld->removeCollisionObject( bt_boxBody[i]);
			bt_boxBody[i] = NULL ;
			
			boxesCount -- ; 
			
		}
	}
}

/// --- Plugin Exection Methods  ---

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
	
	
	CGLContextObj cgl_ctx = [context CGLContextObj];
	
	// Get Context Size and aspect
	width = context.bounds.size.width;
	height = context.bounds.size.height;
	
	aspect = width / height ;
	
	[self initGLScene: cgl_ctx ];
	[self initPhysics];
	 
	
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	Called by Quartz Composer whenever the plug-in instance needs to execute.
	Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	
	The OpenGL context for rendering can be accessed and defined for CGL macros using:
	CGLContextObj cgl_ctx = [context CGLContextObj];
	*/
	
	// --- Set the Gl context ---
	CGLContextObj cgl_ctx = [context CGLContextObj];
	
	// --- Updates ---
	
		// updates view dimensions
		width = self.inputRenderWidth;
		height = self.inputRenderHeight;
		aspect = width/height;
	
		// update world dimensions 
		worldDimWidth = tlMap(self.inputWorldDimWidth, 0., 1., NODE_MAX_DIM*2.f, width - NODE_MAX_DIM*2, true);
		worldDimHeight = tlMap(self.inputWorldDimHeight, 0., 1., NODE_MAX_DIM*2.f, height - NODE_MAX_DIM*2, true);
		worldDimDepth = tlMap(self.inputWorldDimDepth, 0., 1., NODE_MAX_DIM*2.f, width/aspect - NODE_MAX_DIM*2, true);
	
		[self updateWorldDim];
		
	    // update spheres count 
	    [self updateSpheresCount];
	
		// update boxes count 
		[self updateBoxesCount];
	
		// update gravity
		bt_dynamicsWorld->setGravity(btVector3(self.inputGravityX,self.inputGravityY,self.inputGravityZ));
	
	
	// --- Step simulation 
	bt_dynamicsWorld->stepSimulation(1/kRenderFPS,10);
	
	// --- Render GL scene ---
	[self renderGLScene:cgl_ctx];
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	// Clean boundig box bodies
	int i;
	for (i=5; i>=0 ;i--)
	{
		btCollisionObject* obj = bt_dynamicsWorld->getCollisionObjectArray()[i];
		btRigidBody* body = btRigidBody::upcast(obj);
		
		if (body)
		{
			delete body->getMotionState();
			delete body->getCollisionShape();
		}
		bt_dynamicsWorld->removeCollisionObject( obj );
		delete obj;
	}


	// clean spheres bodies
	int _spheresCount = spheresCount;
	
	for(int i=0 ; i<_spheresCount; i++) 
	{
		
		if (bt_sphereBody[i]!= NULL)
		{
			delete bt_sphereBody[i]->getMotionState();
			delete bt_sphereBody[i]->getCollisionShape();
		}
		bt_dynamicsWorld->removeCollisionObject( bt_sphereBody[i]);
		bt_sphereBody[i] = NULL ;
		
		spheresCount -- ; 
		
	}
	
	// clean boxes bodies
	
	int _boxesCount = boxesCount;
	
	
	for(int i=0 ; i<_boxesCount; i++) 
	{
		if (bt_boxBody[i]!= NULL)
		{
			delete bt_boxBody[i]->getMotionState();
			delete bt_boxBody[i]->getCollisionShape();
		}
		bt_dynamicsWorld->removeCollisionObject( bt_boxBody[i]);
		bt_boxBody[i] = NULL ;
		
		boxesCount -- ; 
	}
	
	
	delete bt_dynamicsWorld;
	
	delete bt_solver;
	
	delete bt_broadphase;
	
	delete bt_dispatcher;
	
	delete bt_collisionConfiguration;

}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
	
}

@end
